// UrsSystemTimeServer
//
// Autor: http://UllisRoboterSeite.de
// Doku:  http://bienonline.magix.net/public/esp8266-UrsTime.html
//
// s. UrsTime.h


#ifndef _URSSYSTEMTIMESERVER_h
#define _URSSYSTEMTIMESERVER_h

#include <Arduino.h>
#include <IPAddress.h>
#include <UrsTime.h>
#include <UrsTimeElements.h>
#include <UrsTimeSource.h>

class UrsSystemTimeServer {
protected:
  time_t sysTime = 0;               // Aktuelle UTC-Zeit (POSIX-Format, Sekunden seit 1.1.1970 00:00)

  // Interne Daten fr die Zeitsynchroniation
  uint32_t prevMillis = 0;
  time_t nextSyncTime = 0;
  uint32_t syncInterval = 300;      // Zeit-Synchronisation wird nach der angegebenen Anzahl an Sekunden versucht
  uint32_t syncErrorInterval = 10;  // Sekunden bis zur Wiederholung der Zeitabfrage bei einem Fehler

  uint32_t debugOffset = 0;         // Offset zur Beeinflussung der aktuellen Zeit. Wir zu now() addiert.

  UrsSystemTimeState State = UrsSystemTimeState::timeNotSet; // wird von handle() gesetzt.

  UrsTimeSource& timeSource;
  Timezone& timezone = MEZ;


 

public:
  UrsSystemTimeServer(UrsTimeSource& ts, Timezone& tz = MEZ) : timeSource(ts), timezone(tz)  {}

  void setTime(time_t t); // Zeit neu einstellen

  void setDebugOffsetFromTime(time_t UTC);
  uint32_t getDebugOffset() { return debugOffset; }

  void setSyncInterval(uint32_t interval) { syncInterval = interval; }
  void setSyncErrorInterval(uint32_t interval) { syncErrorInterval = interval; }
  UrsSystemTimeState getState() { return State; }

  time_t now();  // Liefert die aktuelle UTC-Zeit (POSIX-Format, Sekunden seit 1.1.1970 00:00)
     
  // Liefert die aufgelsten Zeitelemente der aktuellen lokalen Zeit gem der eingestellten Zeitzone
  UrsTimeElements localCurrentTime(); 

  int currentOffsetUTC(); // Liefert den aktuellen Timezone-Offset zu UTC in Sekunden

  // Wandelt den TimeStamp (UTC) in aufgelste Zeitelemente gem der eingestellten Zeitzone
  UrsTimeElements toLocalTime(time_t utc);

  bool synchronize(); //Fhrt die Zeit-Synchronisation durch; true, wenn erfogreich

  void handle(); // Fhrt, wenn notwendig, die Zeit-Synchronisation durch
};

#endif

